@@ -1,6 +1,6 @@
 ;;; haskell-cabal.el --- Support for Cabal packages
 
-;; Copyright (C) 2007  Stefan Monnier
+;; Copyright (C) 2007, 2008  Stefan Monnier
 
 ;; Author: Stefan Monnier <monnier@iro.umontreal.ca>
 
@@ -21,7 +21,11 @@
 
 ;;; Commentary:
 
-;; 
+;; Todo:
+
+;; - distinguish continued lines from indented lines.
+;; - indent-line-function.
+;; - outline-minor-mode.
 
 ;;; Code:
 
@@ -68,13 +72,25 @@
 
 (defvar haskell-cabal-mode-syntax-table
   (let ((st (make-syntax-table)))
+    ;; The comment syntax can't be described simply in syntax-table.
+    ;; We could use font-lock-syntactic-keywords, but is it worth it?
+    ;; (modify-syntax-entry ?-  ". 12" st)
+    (modify-syntax-entry ?\n ">" st)
     st))
 
 (defvar haskell-cabal-font-lock-keywords
-  ;; The comment syntax can't be described simply in syntax-table.  We could
-  ;; use font-lock-syntactic-keywords, but is it worth it?
-  '(("^--.*" . font-lock-comment-face)
-    ("^\\([^ :]+\\):" (1 font-lock-keyword-face))))
+  ;; The comment syntax can't be described simply in syntax-table.
+  ;; We could use font-lock-syntactic-keywords, but is it worth it?
+  '(("^[ \t]*--.*" . font-lock-comment-face)
+    ("^ *\\([^ \t:]+\\):" (1 font-lock-keyword-face))
+    ("^\\(Library\\)[ \t]*\\({\\|$\\)" (1 font-lock-keyword-face))
+    ("^\\(Executable\\)[ \t]+\\([^\n \t]*\\)"
+     (1 font-lock-keyword-face) (2 font-lock-function-name-face))
+    ("^\\(Flag\\)[ \t]+\\([^\n \t]*\\)"
+     (1 font-lock-keyword-face) (2 font-lock-constant-face))
+    ("^ *\\(if\\)[ \t]+.*\\({\\|$\\)" (1 font-lock-keyword-face))
+    ("^ *\\(}[ \t]*\\)?\\(else\\)[ \t]*\\({\\|$\\)"
+     (2 font-lock-keyword-face))))
 
 (defvar haskell-cabal-buffers nil
   "List of Cabal buffers.")
@@ -98,13 +114,17 @@
           files)
       (while (and root (equal user (nth 2 (file-attributes root))))
         (if (setq files (directory-files root 'full "\\.cabal\\'"))
-            (throw 'found (find-file-noselect (car files)))
+            ;; Avoid the .cabal directory.
+            (dolist (file files (throw 'found nil))
+              (unless (file-directory-p file)
+                (throw 'found (find-file-noselect file))))
           (if (equal root
                      (setq root (file-name-directory
                                  (directory-file-name root))))
               (setq root nil))))
       nil)))
 
+(autoload 'derived-mode-p "derived")	; Emacs 21
 
 (defun haskell-cabal-buffers-clean (&optional buffer)
   (let ((bufs ()))
@@ -127,7 +147,12 @@
        '(haskell-cabal-font-lock-keywords t t nil nil))
   (add-to-list 'haskell-cabal-buffers (current-buffer))
   (add-hook 'change-major-mode-hook 'haskell-cabal-unregister-buffer nil 'local)
-  (add-hook 'kill-buffer-hook 'haskell-cabal-unregister-buffer nil 'local))
+  (add-hook 'kill-buffer-hook 'haskell-cabal-unregister-buffer nil 'local)
+  (set (make-local-variable 'comment-start) "-- ")
+  (set (make-local-variable 'comment-start-skip) "\\(^[ \t]*\\)--[ \t]*")
+  (set (make-local-variable 'comment-end) "")
+  (set (make-local-variable 'comment-end-skip) "[ 	]*\\(\\s>\\|\n\\)")
+)
 
 (defun haskell-cabal-get-setting (name)
   (save-excursion
