@@ -1,6 +1,6 @@
 ;;; haskell-indent.el --- "semi-intelligent" indentation module for Haskell Mode
 
-;; Copyright 2004, 2005, 2007  Free Software Foundation, Inc.
+;; Copyright 2004, 2005, 2007, 2008, 2009  Free Software Foundation, Inc.
 ;; Copyright 1997-1998  Guy Lapalme
 
 ;; Author: 1997-1998 Guy Lapalme <lapalme@iro.umontreal.ca>
@@ -32,7 +32,7 @@
 ;; Purpose:
 ;;
 ;; To support automatic indentation of Haskell programs using
-;; the layout rule descrived in section 1.5 and appendix B.3 of the
+;; the layout rule described in section 1.5 and appendix B.3 of the
 ;; the Haskell report.  The rationale and the implementation principles
 ;; are described in an article to appear in Journal of Functional Programming.
 ;;   "Dynamic tabbing for automatic indentation with the layout rule"
@@ -125,8 +125,7 @@
           (regexp-opt '("class" "data" "import" "infix" "infixl" "infixr"
                         "instance" "module" "newtype" "primitive" "type") t)
           "\\>")
-  "Regexp describing keywords to complete when standing at the first word
-of a line.")
+  "Regexp for keywords to complete when standing at the first word of a line.")
 
 
 ;; Customizations for different kinds of environments
@@ -153,8 +152,7 @@ Uses free var `haskell-indent-info'."
       (push tmp haskell-indent-info))))
 
 (defun haskell-indent-push-pos (pos &optional name)
-  "Pushes indentation information for the column corresponding to POS
-followed by NAME (if present)."
+  "Push indentation information for POS followed by NAME (if present)."
   (haskell-indent-push-col (haskell-indent-point-to-col pos) name))
 
 ;; (defvar haskell-indent-tab-align nil
@@ -204,8 +202,7 @@ followed by an OFFSET (if present use its value otherwise use
     (back-to-indentation)))
 
 (defun haskell-indent-current-indentation ()
-  "`current-indentation' function but dealing with Bird-style literate
-scripts."
+  "`current-indentation' function dealing with Bird-style literate scripts."
   (if (eq haskell-literate 'bird)
       (save-excursion
         (haskell-indent-back-to-indentation)
@@ -213,8 +210,7 @@ scripts."
     (current-indentation)))
 
 (defun haskell-indent-backward-to-indentation (n)
-  "`backward-to-indentation' function but dealing with Bird-style literate
-scripts."
+  "`backward-to-indentation' function dealing with Bird-style literate scripts."
   (if (eq haskell-literate 'bird)
       (progn
         (forward-line (- n))
@@ -245,8 +241,8 @@ scripts."
     (indent-line-to n)))
 
 (defun haskell-indent-skip-blanks-and-newlines-forward (end)
-  "Skips forward blanks, tabs and newlines until END taking
-account of Bird style literate scripts."
+  "Skip forward blanks, tabs and newlines until END.
+Take account of Bird-style literate scripts."
   (skip-chars-forward " \t\n" end)
   (if (eq haskell-literate 'bird)
       (while (and (bolp) (eq (following-char) ?\>))
@@ -254,8 +250,8 @@ account of Bird style literate scripts."
         (skip-chars-forward " \t\n" end))))
 
 (defun haskell-indent-skip-blanks-and-newlines-backward (start)
-  "Skips backward blanks, tabs and newlines upto START
-taking account of Bird style literate scripts."
+  "Skip backward blanks, tabs and newlines up to START.
+Take account of Bird-style literate scripts."
   (skip-chars-backward " \t\n" start)
   (if (eq haskell-literate 'bird)
       (while (and (eq (current-column) 1)
@@ -266,10 +262,10 @@ taking account of Bird style literate scripts."
 ;; specific functions for literate code
 
 (defun haskell-indent-within-literate-code ()
-  "Checks if point is within a part of literate Haskell code and if so
-returns its start otherwise returns NIL:
-If it is Bird Style, then returns the position of the >
-otherwise returns the ending position \\begin{code}."
+  "Check if point is within a part of literate Haskell code.
+If so, return its start; otherwise return nil:
+If it is Bird-style, then return the position of the >;
+otherwise return the ending position of \\begin{code}."
   (save-excursion
     (case haskell-literate
       (bird
@@ -292,7 +288,7 @@ otherwise returns the ending position \\begin{code}."
 
 (defun haskell-indent-put-region-in-literate (beg end &optional arg)
   "Put lines of the region as a piece of literate code.
-With C-u prefix arg, remove indication that the region is literate code.
+With prefix arg, remove indication that the region is literate code.
 It deals with both Bird style and non Bird-style scripts."
   (interactive "r\nP")
   (unless haskell-literate
@@ -327,6 +323,7 @@ It deals with both Bird style and non Bird-style scripts."
 
 (defcustom haskell-indent-look-past-empty-line t
   "If nil, indentation engine will not look past an empty line for layout points."
+  :group 'haskell-indent
   :type 'boolean)
 
 (defun haskell-indent-start-of-def ()
@@ -396,7 +393,8 @@ Returns the location of the start of the comment, nil otherwise."
   (cond
    ((haskell-indent-empty-line-p) 'empty)
    ((haskell-indent-in-comment (point-min) (point)) 'comment)
-   ((looking-at "\\(\\([a-zA-Z]\\(\\sw\\|'\\)*\\)\\|_\\)[ \t\n]*") 'ident)
+   ((looking-at "\\(\\([[:alpha:]]\\(\\sw\\|'\\)*\\)\\|_\\)[ \t\n]*")
+    'ident)
    ((looking-at "\\(|[^|]\\)[ \t\n]*") 'guard)
    ((looking-at "\\(=[^>=]\\|::\\|->\\|<-\\)[ \t\n]*") 'rhs)
    (t 'other)))
@@ -426,15 +424,20 @@ Returns the location of the start of the comment, nil otherwise."
           contour))))
 
 (defun haskell-indent-next-symbol (end)
-  "Puts point to the next following symbol."
+  "Move point to the next symbol."
   (skip-syntax-forward ")" end)
   (if (< (point) end)
      (progn
        (forward-sexp 1)
        (haskell-indent-skip-blanks-and-newlines-forward end))))
 
+(defun haskell-indent-next-symbol-safe (end)
+  "Puts point to the next following symbol, or to end if there are no more symbols in the sexp."
+  (condition-case errlist (haskell-indent-next-symbol end)
+      (error (goto-char end))))
+
 (defun haskell-indent-separate-valdef (start end)
-  "Returns a list of positions for important parts of a valdef."
+  "Return a list of positions for important parts of a valdef."
   (save-excursion
     (let (valname valname-string aft-valname
                   guard aft-guard
@@ -452,13 +455,13 @@ Returns the location of the start of the comment, nil otherwise."
                   (goto-char (match-end 0)))
               (skip-chars-forward " \t" end)
               (setq valname (point))    ; type = other
-              (haskell-indent-next-symbol end))
+              (haskell-indent-next-symbol-safe end))
             (while (and (< (point) end)
                         (setq type (haskell-indent-type-at-point))
                         (or (memq type '(ident other))))
               (if (null aft-valname)
                   (setq aft-valname (point)))
-              (haskell-indent-next-symbol end))))
+              (haskell-indent-next-symbol-safe end))))
       (if (and (< (point) end) (eq type 'guard)) ; start of a guard
           (progn
             (setq guard (match-beginning 0))
@@ -468,7 +471,7 @@ Returns the location of the start of the comment, nil otherwise."
                         (not (eq type 'rhs)))
               (if (null aft-guard)
                   (setq aft-guard (point)))
-              (haskell-indent-next-symbol end))))
+              (haskell-indent-next-symbol-safe end))))
       (if (and (< (point) end) (eq type 'rhs)) ; start of a rhs
           (progn
             (setq rhs-sign (match-beginning 0))
@@ -486,7 +489,7 @@ Returns the location of the start of the comment, nil otherwise."
 
 
 (defun haskell-indent-guard (start end end-visible indent-info)
-  "Finds indentation information for a line starting with a guard."
+  "Find indentation information for a line starting with a guard."
   (save-excursion
     (let* ((haskell-indent-info indent-info)
            (sep (haskell-indent-separate-valdef start end))
@@ -503,7 +506,7 @@ Returns the location of the start of the comment, nil otherwise."
       haskell-indent-info)))
 
 (defun haskell-indent-rhs (start end end-visible indent-info)
-  "Finds indentation information for a line starting with a rhs."
+  "Find indentation information for a line starting with a rhs."
   (save-excursion
     (let* ((haskell-indent-info indent-info)
            (sep (haskell-indent-separate-valdef start end))
@@ -540,7 +543,7 @@ Returns the location of the start of the comment, nil otherwise."
             "\\)")))
 
 (defun haskell-indent-find-case (test)
-  "Find the index that matches in the decision table."
+  "Find the index that matches TEST in the decision table."
   (if (string-match haskell-indent-decision-table test)
       ;; use the fact that the resulting match-data is a list of the form
       ;; (0 6 [2*(n-1) nil] 0 6) where n is the number of the matching regexp
@@ -549,7 +552,7 @@ Returns the location of the start of the comment, nil otherwise."
     (error "haskell-indent-find-case: impossible case: %s" test)))
 
 (defun haskell-indent-empty (start end end-visible indent-info)
-  "Finds indentation points for an empty line."
+  "Find indentation points for an empty line."
   (save-excursion
     (let* ((haskell-indent-info indent-info)
            (sep (haskell-indent-separate-valdef start end))
@@ -635,7 +638,7 @@ Returns the location of the start of the comment, nil otherwise."
       haskell-indent-info)))
 
 (defun haskell-indent-ident (start end end-visible indent-info)
-  "Finds indentation points for a line starting with an identifier."
+  "Find indentation points for a line starting with an identifier."
   (save-excursion
     (let*
         ((haskell-indent-info indent-info)
@@ -747,7 +750,7 @@ Returns the location of the start of the comment, nil otherwise."
       haskell-indent-info)))
 
 (defun haskell-indent-other (start end end-visible indent-info)
-  "Finds indentation points for a non-empty line starting with something other
+  "Find indentation points for a non-empty line starting with something other
 than an identifier, a guard or rhs."
   (save-excursion
     (let* ((haskell-indent-info indent-info)
@@ -1009,6 +1012,13 @@ OPEN is the start position of the comment in which point is."
                             (haskell-indent-point-to-col (match-end 0)))
                         (haskell-indent-point-to-col (point))))))))))
 
+(defcustom haskell-indent-thenelse 0
+  "If non-nil, \"then\" and \"else\" are indented.
+This is necessary in the \"do\" layout under Haskell-98.
+See http://hackage.haskell.org/trac/haskell-prime/wiki/DoAndIfThenElse"
+  :group 'haskell-indent
+  :type 'integer)
+
 (defun haskell-indent-closing-keyword (start)
   (let ((open (save-excursion
                 (haskell-indent-find-matching-start
@@ -1027,7 +1037,10 @@ OPEN is the start position of the comment in which point is."
       (goto-char open)
       (if (haskell-indent-hanging-p)
           (setq open (haskell-indent-virtual-indentation start))))
-    (list (list (haskell-indent-point-to-col open)))))
+    ;; FIXME: we should try and figure out if the `if' is in a `do' layout
+    ;; before using haskell-indent-thenelse.
+    (list (list (+ (if (memq (char-after) '(?t ?e)) haskell-indent-thenelse 0)
+                   (haskell-indent-point-to-col open))))))
 
 (defcustom haskell-indent-after-keywords
   '(("where" 2 0)
@@ -1049,6 +1062,7 @@ If absent OFFSET defaults to `haskell-indent-offset'.
 
 OFFSET-HANGING is the offset to use in the case where the keyword
 is at the end of an otherwise-non-empty line."
+  :group 'haskell-indent
   :type '(repeat (choice string
                          (cons :tag "" (string :tag "keyword:")
                          (cons :tag "" (integer :tag "offset")
@@ -1076,6 +1090,7 @@ is at the end of an otherwise-non-empty line."
 
 (defcustom haskell-indent-dont-hang '("(")
   "Lexemes that should never be considered as hanging."
+  :group 'haskell-indent
   :type '(repeat string))
 
 (defun haskell-indent-hanging-p ()
@@ -1271,10 +1286,13 @@ We stay in the cycle as long as the TAB key is pressed."
 	(if marker
 	    (goto-char (marker-position marker)))))))
 
+(defun haskell-indent-region (start end)
+  (error "Auto-reindentation of a region is not supported"))
+
 ;;; alignment functions
 
 (defun haskell-indent-shift-columns (dest-column region-stack)
-  "Shifts columns in region-stack to go to DEST-COLUMN.
+  "Shift columns in REGION-STACK to go to DEST-COLUMN.
 Elements of the stack are pairs of points giving the start and end
 of the regions to move."
   (let (reg col diffcol reg-end)
@@ -1407,7 +1425,7 @@ TYPE is either 'guard or 'rhs."
                 (haskell-indent-shift-columns maxcol regstack)))))))
 
 (defun haskell-indent-align-guards-and-rhs (start end)
-  "Align the guards and rhs of functions in the region which must be active."
+  "Align the guards and rhs of functions in the region, which must be active."
   ;; The `start' and `end' args are dummys right now: they're just there so
   ;; we can use the "r" interactive spec which properly signals an error.
   (interactive "*r")
@@ -1446,15 +1464,15 @@ Alignment works only if all guards are to the south-east of their |."
     (haskell-indent-align-def (haskell-indent-mark-active) 'guard)))
 
 (defun haskell-indent-insert-otherwise ()
-  "Insert a guard sign (|) followed by 'otherwise' and align the
-previous guards of the current function."
+  "Insert a guard sign (|) followed by `otherwise'.
+Also align the previous guards of the current function."
   (interactive "*")
   (haskell-indent-insert-guard "otherwise")
   (haskell-indent-insert-equal))
 
 (defun haskell-indent-insert-where ()
-  "Insert and a where keyword at point and indent the resulting
-line with an indentation cycle."
+  "Insert a where keyword at point and indent resulting line.
+One indentation cycle is used."
   (interactive "*")
   (insert "where ")
   (haskell-indent-cycle))
@@ -1463,8 +1481,7 @@ line with an indentation cycle."
 ;;; haskell-indent-mode
 
 (defvar haskell-indent-mode nil
-  "Indicates if the semi-intelligent Haskell indentation mode is in effect
-in the current buffer.")
+  "Non-nil if the semi-intelligent Haskell indentation mode is in effect.")
 (make-variable-buffer-local 'haskell-indent-mode)
 
 (defvar haskell-indent-map
@@ -1476,7 +1493,8 @@ in the current buffer.")
     (define-key map [?\C-c ?\C-=] 'haskell-indent-insert-equal)
     (define-key map [?\C-c ?\C-|] 'haskell-indent-insert-guard)
     ;; Alternate binding, in case C-c C-| is too inconvenient to type.
-    (define-key map [?\C-c ?\C-g] 'haskell-indent-insert-guard)
+    ;; Duh, C-g is a special key, let's not use it here.
+    ;; (define-key map [?\C-c ?\C-g] 'haskell-indent-insert-guard)
     (define-key map [?\C-c ?\C-o] 'haskell-indent-insert-otherwise)
     (define-key map [?\C-c ?\C-w] 'haskell-indent-insert-where)
     (define-key map [?\C-c ?\C-.] 'haskell-indent-align-guards-and-rhs)
@@ -1484,8 +1502,9 @@ in the current buffer.")
     map))
 
 (defun turn-on-haskell-indent ()
-  "Turn on ``intelligent'' haskell indentation mode."
+  "Turn on ``intelligent'' Haskell indentation mode."
   (set (make-local-variable 'indent-line-function) 'haskell-indent-cycle)
+  (set (make-local-variable 'indent-region-function) 'haskell-indent-region)
   (setq haskell-indent-mode t)
   ;; Activate our keymap.
   (let ((map (current-local-map)))
@@ -1507,8 +1526,7 @@ in the current buffer.")
   (run-hooks 'haskell-indent-hook))
 
 (defun turn-off-haskell-indent ()
-  "Turn off ``intelligent'' haskell indentation mode that deals with
-the layout rule of Haskell."
+  "Turn off ``intelligent'' Haskell indentation mode."
   (kill-local-variable 'indent-line-function)
   ;; Remove haskell-indent-map from the local map.
   (let ((map (current-local-map)))
@@ -1527,11 +1545,12 @@ the layout rule of Haskell."
 
 ;;;###autoload
 (defun haskell-indent-mode (&optional arg)
-  "``intelligent'' Haskell indentation mode that deals with
-the layout rule of Haskell.  \\[haskell-indent-cycle] starts the cycle
-which proposes new possibilities as long as the TAB key is pressed.
-Any other key or mouse click terminates the cycle and is interpreted
-except for RET which merely exits the cycle.
+  "``Intelligent'' Haskell indentation mode.
+This deals with the layout rule of Haskell.
+\\[haskell-indent-cycle] starts the cycle which proposes new
+possibilities as long as the TAB key is pressed.  Any other key
+or mouse click terminates the cycle and is interpreted except for
+RET which merely exits the cycle.
 Other special keys are:
     \\[haskell-indent-insert-equal]
       inserts an =
@@ -1547,12 +1566,6 @@ these functions also align the guards and rhs of the current definition
     \\[haskell-indent-put-region-in-literate]
       makes the region a piece of literate code in a literate script
 
-Note: \\[indent-region] which applies \\[haskell-indent-cycle] for each line
-of the region also works but it stops and asks for any line having more
-than one possible indentation.
-Use TAB to cycle until the right indentation is found and then RET to go the
-next line to indent.
-
 Invokes `haskell-indent-hook' if not nil."
   (interactive "P")
   (setq haskell-indent-mode
