@@ -1,6 +1,6 @@
 ;;; haskell-mode.el --- A Haskell editing mode    -*-coding: iso-8859-1;-*-
 
-;; Copyright (C) 2003, 2004, 2005, 2006, 2007  Free Software Foundation, Inc
+;; Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008  Free Software Foundation, Inc
 ;; Copyright (C) 1992, 1997-1998 Simon Marlow, Graeme E Moss, and Tommy Thorn
 
 ;; Authors: 1992      Simon Marlow
@@ -9,14 +9,14 @@
 ;;          2001-2002 Reuben Thomas (>=v1.4)
 ;;          2003      Dave Love <fx@gnu.org>
 ;; Keywords: faces files Haskell
-;; Version: v2_3
+;; Version: v2.7.0
 ;; URL: http://www.haskell.org/haskell-mode/
 
 ;; This file is not part of GNU Emacs.
 
 ;; This file is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
+;; the Free Software Foundation; either version 3, or (at your option)
 ;; any later version.
 
 ;; This file is distributed in the hope that it will be useful,
@@ -46,6 +46,9 @@
 ;; `haskell-doc', Hans-Wolfgang Loidl
 ;;   Echoes types of functions or syntax of keywords when the cursor is idle.
 ;;
+;; `haskell-indentation', Kristof Bastiaensen
+;;   Intelligent semi-automatic indentation, mark two.
+;;
 ;; `haskell-indent', Guy Lapalme
 ;;   Intelligent semi-automatic indentation.
 ;;
@@ -62,49 +65,6 @@
 ;; This mode supports full Haskell 1.4 including literate scripts.
 ;; In some versions of (X)Emacs it may only support Latin-1, not Unicode.
 ;;
-;; Installation:
-;; 
-;; Put in your ~/.emacs:
-;;
-;;    (setq auto-mode-alist
-;;          (append auto-mode-alist
-;;                  '(("\\.[hg]s$"  . haskell-mode)
-;;                    ("\\.hi$"     . haskell-mode)
-;;                    ("\\.l[hg]s$" . literate-haskell-mode))))
-;;
-;;    (autoload 'haskell-mode "haskell-mode"
-;;       "Major mode for editing Haskell scripts." t)
-;;    (autoload 'literate-haskell-mode "haskell-mode"
-;;       "Major mode for editing literate Haskell scripts." t)
-;;
-;; with `haskell-mode.el' accessible somewhere on the load-path.
-;; To add a directory `~/lib/emacs' (for example) to the load-path,
-;; add the following to .emacs:
-;;
-;;    (setq load-path (cons "~/lib/emacs" load-path))
-;;
-;; To turn any of the supported modules on for all buffers, add the
-;; appropriate line(s) to .emacs:
-;;
-;;    (add-hook 'haskell-mode-hook 'turn-on-haskell-decl-scan)
-;;    (add-hook 'haskell-mode-hook 'turn-on-haskell-doc-mode)
-;;    (add-hook 'haskell-mode-hook 'turn-on-haskell-indent)
-;;    (add-hook 'haskell-mode-hook 'turn-on-haskell-simple-indent)
-;;
-;; Make sure the module files are also on the load-path.  Note that
-;; the two indentation modules are mutually exclusive: Use only one.
-;;
-;;
-;; Customisation:
-;;
-;; Set the value of `haskell-literate-default' to your preferred
-;; literate style: 'bird or 'latex, within .emacs as follows:
-;;
-;;    (setq haskell-literate-default 'latex)
-;;
-;; Also see the customisations of the modules.
-;;
-;;
 ;; History:
 ;;
 ;; This mode is based on an editing mode by Simon Marlow 11/1/92
@@ -118,6 +78,9 @@
 ;; version of Emacs you are using, and a small example of the problem
 ;; or suggestion.
 ;;
+;; Version 1.5
+;;   Added autoload for haskell-indentation
+;;
 ;; Version 1.43:
 ;;   Various tweaks to doc strings and customization support from
 ;;   Ville Skytt <scop@xemacs.org>.
@@ -135,7 +98,7 @@
 ;;
 ;; Version 1.3:
 ;;   The literate or non-literate style of a buffer is now indicated
-;;   by just the variable haskell-literate: nil, 'bird, or 'latex.
+;;   by just the variable haskell-literate: nil, `bird', or `tex'.
 ;;   For literate buffers with ambiguous style, the value of
 ;;   haskell-literate-default is used.
 ;;
@@ -195,11 +158,18 @@
 ;;; Code:
 
 (eval-when-compile (require 'cl))
+(eval-when-compile
+  ;; Emacs 21 defines `values' as a (run-time) alias for list.
+  ;; Don't maerge this with the pervious clause.
+  (if (string-match "values"
+		    (pp (byte-compile (lambda () (values t)))))
+      (defsubst values (&rest values)
+	values)))
 
 ;; All functions/variables start with `(literate-)haskell-'.
 
 ;; Version of mode.
-(defconst haskell-version "v2_3"
+(defconst haskell-version "v2.7.0"
   "`haskell-mode' version number.")
 (defun haskell-version ()
   "Echo the current version of `haskell-mode' in the minibuffer."
@@ -211,11 +181,18 @@
   :group 'languages
   :prefix "haskell-")
 
+;; Set load-path
+;;;###autoload
+(add-to-list 'load-path
+   (or (file-name-directory load-file-name) (car load-path)))
+
 ;; Set up autoloads for the modules we supply
 (autoload 'turn-on-haskell-decl-scan "haskell-decl-scan"
   "Turn on Haskell declaration scanning." t)
 (autoload 'turn-on-haskell-doc-mode "haskell-doc"
   "Turn on Haskell Doc minor mode." t)
+(autoload 'turn-on-haskell-indentation "haskell-indentation"
+  "Turn on advanced Haskell indentation." t)
 (autoload 'turn-on-haskell-indent "haskell-indent"
   "Turn on Haskell indentation." t)
 (autoload 'turn-on-haskell-simple-indent "haskell-simple-indent"
@@ -230,34 +207,28 @@
 (defun turn-on-haskell-font-lock ()
   (turn-on-font-lock)
   (message "turn-on-haskell-font-lock is obsolete.  Use turn-on-font-lock instead."))
-(defun turn-on-haskell-hugs ()
-  (message "haskell-hugs is obsolete.")
-  (load "haskell-hugs")
-  (turn-on-haskell-hugs))
-(defun turn-on-haskell-ghci ()
-  (message "haskell-ghci is obsolete.")
-  (load "haskell-ghci")
-  (turn-on-haskell-ghci))
+(defun turn-on-haskell-hugs () (message "haskell-hugs is obsolete."))
+(defun turn-on-haskell-ghci () (message "haskell-ghci is obsolete."))
 
 
 ;; Are we looking at a literate script?
 (defvar haskell-literate nil
   "*If not nil, the current buffer contains a literate Haskell script.
-Possible values are: `bird' and `latex', for Bird-style and LaTeX-style
+Possible values are: `bird' and `tex', for Bird-style and LaTeX-style
 literate scripts respectively.  Set by `haskell-mode' and
-`literate-haskell-mode'.  For an ambiguous literate buffer -- ie. does
+`literate-haskell-mode'.  For an ambiguous literate buffer -- i.e. does
 not contain either \"\\begin{code}\" or \"\\end{code}\" on a line on
 its own, nor does it contain \">\" at the start of a line -- the value
-of `haskell-literate-default' is used.
-
-Always buffer-local.")
+of `haskell-literate-default' is used.")
 (make-variable-buffer-local 'haskell-literate)
+(put 'haskell-literate 'safe-local-variable 'symbolp)
 ;; Default literate style for ambiguous literate buffers.
 (defcustom haskell-literate-default 'bird
-  "*Default value for `haskell-literate'.
+  "Default value for `haskell-literate'.
 Used if the style of a literate buffer is ambiguous.  This variable should
 be set to the preferred literate style."
-  :type '(choice (const bird) (const latex) (const nil)))
+  :group 'haskell
+  :type '(choice (const bird) (const tex) (const nil)))
 
 ;; Mode maps.
 (defvar haskell-mode-map
@@ -268,8 +239,10 @@ be set to the preferred literate style."
     ;; (define-key map [?\C-c ?\C-r] 'inferior-haskell-send-region)
     (define-key map [?\C-c ?\C-z] 'switch-to-haskell)
     (define-key map [?\C-c ?\C-l] 'inferior-haskell-load-file)
-    ;; Non standard in other inferior-modes, but traditional in haskell.
-    (define-key map [?\C-c ?\C-r] 'inferior-haskell-reload-file)
+    ;; I think it makes sense to bind inferior-haskell-load-and-run to C-c
+    ;; C-r, but since it used to be bound to `reload' until June 2007, I'm
+    ;; going to leave it out for now.
+    ;; (define-key map [?\C-c ?\C-r] 'inferior-haskell-load-and-run)
     (define-key map [?\C-c ?\C-b] 'switch-to-haskell)
     ;; (define-key map [?\C-c ?\C-s] 'inferior-haskell-start-process)
     ;; That's what M-; is for.
@@ -278,6 +251,11 @@ be set to the preferred literate style."
     (define-key map (kbd "C-c C-t") 'inferior-haskell-type)
     (define-key map (kbd "C-c C-i") 'inferior-haskell-info)
     (define-key map (kbd "C-c M-.") 'inferior-haskell-find-definition)
+    (define-key map (kbd "C-c C-d") 'inferior-haskell-find-haddock)
+
+    (define-key map [?\C-c ?\C-v] 'haskell-check)
+
+    (define-key map [remap delete-indentation] 'haskell-delete-indentation)
     map)
   "Keymap used in Haskell mode.")
 
@@ -340,7 +318,7 @@ be set to the preferred literate style."
 	   (if (consp k)
 	       (setq i (car k)
 		     lim (cdr k))
-	     (setq i k 
+	     (setq i k
 		   lim k))
 	   (while (<= i lim)
 	     (when (> i 127)
@@ -350,42 +328,80 @@ be set to the preferred literate style."
     
     (modify-syntax-entry ?\` "$`" table)
     (modify-syntax-entry ?\\ "\\" table)
-    (mapcar (lambda (x)
-	      (modify-syntax-entry x "_" table))
-	    ;; Some of these are actually OK by default.
-	    "!#$%&*+./:<=>?@^|~")
+    (mapc (lambda (x)
+            (modify-syntax-entry x "_" table))
+          ;; Some of these are actually OK by default.
+          "!#$%&*+./:<=>?@^|~")
     (unless (featurep 'mule)
       ;; Non-ASCII syntax should be OK, at least in Emacs.
-      (mapcar (lambda (x)
-		(modify-syntax-entry x "_" table))
-	      (concat ""
-		      ""))
-      (mapcar (lambda (x)
-		(modify-syntax-entry x "w" table))
-	      (concat ""
-		      ""
-		      ""
-		      "")))
+      (mapc (lambda (x)
+              (modify-syntax-entry x "_" table))
+            (concat ""
+                    ""))
+      (mapc (lambda (x)
+              (modify-syntax-entry x "w" table))
+            (concat ""
+                    ""
+                    ""
+                    "")))
     table)
   "Syntax table used in Haskell mode.")
 
 (defun haskell-ident-at-point ()
-  "Return the identifier under point, or nil if none found."
+  "Return the identifier under point, or nil if none found.
+May return a qualified name."
   (save-excursion
-    (if (looking-at "\\s_")
-        (buffer-substring-no-properties
-         (progn (skip-syntax-backward "_") (point))
-         (progn (skip-syntax-forward "_") (point)))
-      (buffer-substring-no-properties
-       (progn (skip-syntax-backward "w'") (skip-syntax-forward "'") (point))
-       (progn (skip-syntax-forward "w'") (point))))))
+    (let ((case-fold-search nil))
+      (multiple-value-bind (start end)
+          (if (looking-at "\\s_")
+              (values (progn (skip-syntax-backward "_") (point))
+                      (progn (skip-syntax-forward "_") (point)))
+            (values
+             (progn (skip-syntax-backward "w'")
+                    (skip-syntax-forward "'") (point))
+             (progn (skip-syntax-forward "w'") (point))))
+        ;; If we're looking at a module ID that qualifies further IDs, add
+        ;; those IDs.
+        (goto-char start)
+        (while (and (looking-at "[[:upper:]]") (eq (char-after end) ?.)
+                    ;; It's a module ID that qualifies further IDs.
+                    (goto-char (1+ end))
+                    (save-excursion
+                      (when (not (zerop (skip-syntax-forward
+                                         (if (looking-at "\\s_") "_" "w'"))))
+                        (setq end (point))))))
+        ;; If we're looking at an ID that's itself qualified by previous
+        ;; module IDs, add those too.
+        (goto-char start)
+        (if (eq (char-after) ?.) (forward-char 1)) ;Special case for "."
+        (while (and (eq (char-before) ?.)
+                    (progn (forward-char -1)
+                           (not (zerop (skip-syntax-backward "w'"))))
+                    (skip-syntax-forward "'")
+                    (looking-at "[[:upper:]]"))
+          (setq start (point)))
+        ;; This is it.
+        (buffer-substring-no-properties start end)))))
+
+(defun haskell-delete-indentation (&optional arg)
+  "Like `delete-indentation' but ignoring Bird-style \">\"."
+  (interactive "*P")
+  (let ((fill-prefix (or fill-prefix (if (eq haskell-literate 'bird) ">"))))
+    (delete-indentation arg)))
 
 ;; Various mode variables.
 
 (defcustom haskell-mode-hook nil
-  "Hook run after entering Haskell mode."
+  "Hook run after entering Haskell mode.
+Do not select more than one of the three indentation modes."
   :type 'hook
-  :options '(turn-on-haskell-indent turn-on-font-lock turn-on-eldoc-mode
+  :group 'haskell
+  :options `(turn-on-haskell-indent turn-on-haskell-indentation
+	     turn-on-font-lock
+	     ,(if (boundp 'eldoc-documentation-function)
+		  'turn-on-eldoc-mode
+		'turn-on-haskell-doc-mode) ; Emacs 21
+	     turn-on-simple-indent turn-on-haskell-doc-mode
 	     imenu-add-menubar-index))
 
 (defvar eldoc-print-current-symbol-info-function)
@@ -393,16 +409,12 @@ be set to the preferred literate style."
 ;; The main mode functions
 ;;;###autoload
 (define-derived-mode haskell-mode fundamental-mode "Haskell"
-  "Major mode for editing Haskell programs.  Last adapted for Haskell 1.4.
+  "Major mode for editing Haskell programs.
 Blank lines separate paragraphs, comments start with `-- '.
-
-\\<haskell-mode-map>\\[indent-for-comment] will place a comment at an appropriate place on the current line.
-\\[comment-region] comments (or with prefix arg, uncomments) each line in the region.
-
-Literate scripts are supported via `literate-haskell-mode'.  The
-variable `haskell-literate' indicates the style of the script in the
-current buffer.  See the documentation on this variable for more
-details.
+\\<haskell-mode-map>
+Literate scripts are supported via `literate-haskell-mode'.
+The variable `haskell-literate' indicates the style of the script in the
+current buffer.  See the documentation on this variable for more details.
 
 Modules can hook in via `haskell-mode-hook'.  The following modules
 are supported with an `autoload' command:
@@ -413,6 +425,9 @@ are supported with an `autoload' command:
    `haskell-doc', Hans-Wolfgang Loidl
      Echoes types of functions or syntax of keywords when the cursor is idle.
 
+   `haskell-indentation', Kristof Bastiaensen
+     Intelligent semi-automatic indentation Mk2
+
    `haskell-indent', Guy Lapalme
      Intelligent semi-automatic indentation.
 
@@ -420,16 +435,19 @@ are supported with an `autoload' command:
      Simple indentation.
 
 Module X is activated using the command `turn-on-X'.  For example,
-`haskell-font-lock' is activated using `turn-on-haskell-font-lock'.
-For more information on a module, see the help for its `turn-on-X'
+`haskell-indent' is activated using `turn-on-haskell-indent'.
+For more information on a module, see the help for its `X-mode'
 function.  Some modules can be deactivated using `turn-off-X'.  (Note
 that `haskell-doc' is irregular in using `turn-(on/off)-haskell-doc-mode'.)
 
 Use `haskell-version' to find out what version this is.
 
-Invokes `haskell-mode-hook' if not nil."
+Invokes `haskell-mode-hook'."
   (set (make-local-variable 'paragraph-start) (concat "^$\\|" page-delimiter))
   (set (make-local-variable 'paragraph-separate) paragraph-start)
+  (set (make-local-variable 'fill-paragraph-function) 'haskell-fill-paragraph)
+  ;; (set (make-local-variable 'adaptive-fill-function) 'haskell-adaptive-fill)
+  (set (make-local-variable 'adaptive-fill-mode) nil)
   (set (make-local-variable 'comment-start) "-- ")
   (set (make-local-variable 'comment-padding) 0)
   (set (make-local-variable 'comment-start-skip) "[-{]-[ \t]*")
@@ -459,6 +477,30 @@ Invokes `haskell-mode-hook' if not nil."
   (set (make-local-variable 'tab-width) 8)
   (setq haskell-literate nil))
 
+(defun in-comment () (nth 4 (syntax-ppss)))
+
+(defun haskell-fill-paragraph (justify)
+  (save-excursion
+    ;; We don't want to reflow code.
+    (unless (in-comment)
+      (end-of-line)) ; Try to get inside a comment
+    (if (in-comment) nil t)))
+
+;; (defun haskell-adaptive-fill ()
+;;   ;; We want to use "--  " as the prefix of "-- |", etc.
+;;   (let* ((line-end (save-excursion (end-of-line) (point)))
+;;          (line-start (point)))
+;;     (save-excursion
+;;       (unless (in-comment)
+;;         ;; Try to find the start of a comment. We only fill comments.
+;;         (search-forward-regexp comment-start-skip line-end t))
+;;       (when (in-comment)
+;;         (let ();(prefix-start (point)))
+;;           (skip-syntax-forward "^w")
+;;           (make-string (- (point) line-start) ?\s))))))
+          
+
+
 ;;;###autoload
 (define-derived-mode literate-haskell-mode haskell-mode "LitHaskell"
   "As `haskell-mode' but for literate scripts."
@@ -466,15 +508,110 @@ Invokes `haskell-mode-hook' if not nil."
         (save-excursion
           (goto-char (point-min))
           (cond
-           ((re-search-forward "^\\\\\\(begin\\|end\\){code}$" nil t) 'latex)
+           ((re-search-forward "^\\\\\\(begin\\|end\\){code}$" nil t) 'tex)
            ((re-search-forward "^>" nil t) 'bird)
-           (t haskell-literate-default)))))
+           (t haskell-literate-default))))
+  (if (eq haskell-literate 'bird)
+      ;; fill-comment-paragraph isn't much use there, and even gets confused
+      ;; by the syntax-table text-properties we add to mark the first char
+      ;; of each line as a comment-starter.
+      (set (make-local-variable 'fill-paragraph-handle-comment) nil))
+  (set (make-local-variable 'mode-line-process)
+       '("/" (:eval (symbol-name haskell-literate)))))
 
 ;;;###autoload(add-to-list 'auto-mode-alist '("\\.\\(?:[gh]s\\|hi\\)\\'" . haskell-mode))
 ;;;###autoload(add-to-list 'auto-mode-alist '("\\.l[gh]s\\'" . literate-haskell-mode))
-;;;###autoload(add-hook 'haskell-mode-hook 'turn-on-haskell-doc-mode)
-;;;###autoload(add-hook 'haskell-mode-hook 'turn-on-haskell-indent)
 
+(defcustom haskell-hoogle-command
+  (if (executable-find "hoogle") "hoogle")
+  "Name of the command to use to query Hoogle.
+If nil, use the Hoogle web-site."
+  :group 'haskell
+  :type '(choice (const :tag "Use Web-site" nil)
+                 string))
+
+;;;###autoload
+(defun haskell-hoogle (query)
+  "Do a Hoogle search for QUERY."
+  (interactive
+   (let ((def (haskell-ident-at-point)))
+     (if (and def (symbolp def)) (setq def (symbol-name def)))
+     (list (read-string (if def
+                            (format "Hoogle query (default %s): " def)
+                          "Hoogle query: ")
+                        nil nil def))))
+  (if (null haskell-hoogle-command)
+      (browse-url (format "http://haskell.org/hoogle/?q=%s" query))
+    (if (fboundp 'help-setup-xref)
+        (help-setup-xref (list 'haskell-hoogle query) (interactive-p)))
+    (with-output-to-temp-buffer
+        (if (fboundp 'help-buffer) (help-buffer) "*Help*")
+      (with-current-buffer standard-output
+        (start-process "hoogle" (current-buffer) haskell-hoogle-command
+                       query)))))
+
+;;;###autoload
+(defalias 'hoogle 'haskell-hoogle)
+
+;;;###autoload
+(defun haskell-hayoo (query)
+  "Do a Hayoo search for QUERY."
+  (interactive
+   (let ((def (haskell-ident-at-point)))
+     (if (and def (symbolp def)) (setq def (symbol-name def)))
+     (list (read-string (if def
+                            (format "Hayoo query (default %s): " def)
+                          "Hayoo query: ")
+                        nil nil def))))
+  (browse-url (format "http://holumbus.fh-wedel.de/hayoo/hayoo.html?query=%s" query)))
+
+;;;###autoload
+(defalias 'hayoo 'haskell-hayoo)
+
+(defcustom haskell-check-command "hlint"
+  "*Command used to check a Haskell file."
+  :group 'haskell
+  :type '(choice (const "hlint")
+		 (const "ghc -fno-code")
+		 (string :tag "Other command")))
+
+(defvar haskell-saved-check-command nil
+  "Internal use.")
+
+;; Like Python.  Should be abstracted, sigh.
+(defun haskell-check (command)
+  "Check a Haskell file (default current buffer's file).
+Runs COMMAND, a shell command, as if by `compile'.
+See `haskell-check-command' for the default."
+  (interactive
+   (list (read-string "Checker command: "
+		      (or haskell-saved-check-command
+			  (concat haskell-check-command " "
+				  (let ((name (buffer-file-name)))
+				    (if name
+					(file-name-nondirectory name))))))))
+  (setq haskell-saved-check-command command)
+  (require 'compile)
+  (save-some-buffers (not compilation-ask-about-save) nil)
+  (if (fboundp 'compilation-start)
+      (compilation-start command)
+    (compile-internal command "No more errors")))
+
+(autoload 'flymake-init-create-temp-buffer-copy "flymake")
+
+(defun haskell-flymake-init ()
+  "Flymake init function for Haskell.
+To be added to `flymake-init-create-temp-buffer-copy'."
+  (let ((checker-elts (split-string haskell-saved-check-command)))
+    (list (car checker-elts)
+	  (append (cdr checker-elts)
+		  (list (flymake-init-create-temp-buffer-copy
+			 'flymake-create-temp-inplace))))))
+
+(eval-after-load "flymake"
+  '(add-to-list 'flymake-allowed-file-name-masks
+		'("\\.l?hs\\'" haskell-flymake-init)))
+
 ;; Provide ourselves:
 
 (provide 'haskell-mode)
