@@ -1,6 +1,6 @@
 ;;; inf-haskell.el --- Interaction with an inferior Haskell process.
 
-;; Copyright (C) 2004, 2005, 2006, 2007  Free Software Foundation, Inc.
+;; Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Free Software Foundation, Inc.
 
 ;; Author: Stefan Monnier <monnier@iro.umontreal.ca>
 ;; Keywords: Haskell
@@ -38,6 +38,29 @@
 (require 'haskell-mode)
 (eval-when-compile (require 'cl))
 
+;; XEmacs compatibility.
+
+(unless (fboundp 'subst-char-in-string)
+  (defun subst-char-in-string (fromchar tochar string &optional inplace)
+    ;; This is Haskell-mode, we don't want no stinkin' `aset'.
+    (apply 'string (mapcar (lambda (c) (if (eq c fromchar) tochar c)) string))))
+
+(unless (fboundp 'make-temp-file)
+  (defun make-temp-file (prefix &optional dir-flag)
+    (catch 'done
+      (while t
+        (let ((f (make-temp-name (expand-file-name prefix (temp-directory)))))
+          (condition-case ()
+              (progn
+                (if dir-flag (make-directory f)
+                  (write-region "" nil f nil 'silent nil))
+                (throw 'done f))
+            (file-already-exists t)))))))
+
+(unless (fboundp 'replace-regexp-in-string)
+  (defun replace-regexp-in-string (regexp rep string)
+    (replace-in-string string regexp rep)))
+
 ;; Here I depart from the inferior-haskell- prefix.
 ;; Not sure if it's a good idea.
 (defcustom haskell-program-name
@@ -56,7 +79,7 @@ The command can include arguments."
   :type '(choice string (repeat string)))
 
 (defconst inferior-haskell-info-xref-re
-  "\t-- Defined at \\(.+\\):\\([0-9]+\\):\\([0-9]+\\)$")
+  "\t-- Defined at \\(.+\\):\\([0-9]+\\):\\([0-9]+\\)\\(?:-\\([0-9]+\\)\\)?$")
 
 (defconst inferior-haskell-module-re
   "\t-- Defined in \\(.+\\)$"
@@ -91,20 +114,23 @@ The command can include arguments."
           ("imported from .* at \\(.+\\):\\([0-9]+\\):\\([0-9]+\\)-\\([0-9]+\\)$"
            1 2 (3 . 4) 0)
           ;; Info xrefs.
-          (,inferior-haskell-info-xref-re 1 2 3 0))))
+          (,inferior-haskell-info-xref-re 1 2 (3 . 4) 0))))
   "Regexps for error messages generated by inferior Haskell processes.
 The format should be the same as for `compilation-error-regexp-alist'.")
 
 (defcustom inferior-haskell-find-project-root t
   "If non-nil, try and find the project root directory of this file.
 This will either look for a Cabal file or a \"module\" statement in the file."
+  :group 'haskell
   :type 'boolean)
 
 (define-derived-mode inferior-haskell-mode comint-mode "Inf-Haskell"
   "Major mode for interacting with an inferior Haskell process."
   (set (make-local-variable 'comint-prompt-regexp)
-       "^\\*?[A-Z][\\._a-zA-Z0-9]*\\( \\*?[A-Z][\\._a-zA-Z0-9]*\\)*> ")
+       ;; Whay the backslash in [\\._[:alnum:]]?
+       "^\\*?[[:upper:]][\\._[:alnum:]]*\\(?: \\*?[[:upper:]][\\._[:alnum:]]*\\)*> ")
   (set (make-local-variable 'comint-input-autoexpand) nil)
+  (add-hook 'comint-output-filter-functions 'inferior-haskell-spot-prompt nil t)
 
   ;; Setup directory tracking.
   (set (make-local-variable 'shell-cd-regexp) ":cd")
@@ -196,22 +222,41 @@ setting up the inferior-haskell buffer."
   :type 'boolean
   :group 'haskell)
 
-(defun inferior-haskell-wait-for-prompt (proc)
+(defvar inferior-haskell-seen-prompt nil)
+(make-variable-buffer-local 'inferior-haskell-seen-prompt)
+
+(defun inferior-haskell-spot-prompt (string)
+  (let ((proc (get-buffer-process (current-buffer))))
+    (when proc
+      (save-excursion
+        (goto-char (process-mark proc))
+        (if (re-search-backward comint-prompt-regexp
+                                (line-beginning-position) t)
+            (setq inferior-haskell-seen-prompt t))))))
+
+(defun inferior-haskell-wait-for-prompt (proc &optional timeout)
   "Wait until PROC sends us a prompt.
 The process PROC should be associated to a comint buffer."
   (with-current-buffer (process-buffer proc)
     (while (progn
              (goto-char comint-last-input-end)
-             (and (not (re-search-forward comint-prompt-regexp nil t))
-                  (accept-process-output proc))))))
+             (not (or inferior-haskell-seen-prompt
+                      (setq inferior-haskell-seen-prompt
+                            (re-search-forward comint-prompt-regexp nil t))
+                      (not (accept-process-output proc timeout))))))
+    (unless inferior-haskell-seen-prompt
+      (error "Can't find the prompt"))))
 
 (defvar inferior-haskell-cabal-buffer nil)
 
 (defun inferior-haskell-cabal-of-buf (buf)
   (require 'haskell-cabal)
   (with-current-buffer buf
-    (or inferior-haskell-cabal-buffer
-        (and (not (local-variable-p 'inferior-haskell-cabal-buffer))
+    (or (and (buffer-live-p inferior-haskell-cabal-buffer)
+             inferior-haskell-cabal-buffer)
+        (and (not (local-variable-p 'inferior-haskell-cabal-buffer
+                                    ;; XEmacs needs this argument.
+                                    (current-buffer)))
              (set (make-local-variable 'inferior-haskell-cabal-buffer)
                   (haskell-cabal-find-file))))))
 
@@ -237,7 +282,7 @@ The process PROC should be associated to a comint buffer."
             (goto-char (point-min))
             (let ((case-fold-search nil))
               (when (re-search-forward
-                     "^module[ \t]+\\([^- \t\n]+\\.[^- \t\n]+\\)[ \t]+where\\>" nil t)
+                     "^module[ \t]+\\([^- \t\n]+\\.[^- \t\n]+\\)[ \t]+" nil t)
                 (let* ((dir default-directory)
                        (module (match-string 1))
                        (pos 0))
@@ -261,8 +306,8 @@ The process PROC should be associated to a comint buffer."
                     (message "Ignoring inconsistent `module' info: %s in %s"
                              module buffer-file-name)
                     nil)))))))))
-                    
-                    
+
+
 
 ;;;###autoload
 (defun inferior-haskell-load-file (&optional reload)
@@ -288,7 +333,11 @@ If prefix arg \\[universal-argument] is given, just reload the previous file."
              proc (concat ":cd " default-directory)))
           (setq file (file-relative-name file)))
 	(inferior-haskell-send-command
-         proc (if reload ":reload" (concat ":load \"" file "\"")))
+         proc (if reload ":reload"
+                (concat ":load \""
+                        ;; Espace the backslashes that may occur in file names.
+                        (replace-regexp-in-string "[\\\"]" "\\\\\&" file)
+                        "\"")))
 	;; Move the parsing-end marker *after* sending the command so
 	;; that it doesn't point just to the insertion point.
 	;; Otherwise insertion may move the marker (if done with
@@ -319,9 +368,16 @@ If prefix arg \\[universal-argument] is given, just reload the previous file."
         inferior-haskell-run-command
       (read-string "Command to run: " nil nil inferior-haskell-run-command))))
   (setq inferior-haskell-run-command command)
-  (inferior-haskell-load-file)
-  ;; FIXME: if the :load encountered errors, we should not `run'.
-  (inferior-haskell-send-command (inferior-haskell-process) command))
+  (let* ((inferior-haskell-errors nil)
+         (neh (lambda () (setq inferior-haskell-errors t))))
+    (unwind-protect
+        (let ((inferior-haskell-wait-and-jump t))
+          (add-hook 'next-error-hook neh)
+          (inferior-haskell-load-file))
+      (remove-hook 'next-error-hook neh))
+    (unless inferior-haskell-errors
+      (inferior-haskell-send-command (inferior-haskell-process) command)
+      (switch-to-haskell))))
 
 (defun inferior-haskell-send-command (proc str)
   (setq str (concat str "\n"))
@@ -330,6 +386,7 @@ If prefix arg \\[universal-argument] is given, just reload the previous file."
     (goto-char (process-mark proc))
     (insert-before-markers str)
     (move-marker comint-last-input-end (point))
+    (setq inferior-haskell-seen-prompt nil)
     (comint-send-string proc str)))
 
 (defun inferior-haskell-reload-file ()
@@ -358,8 +415,8 @@ The returned info is cached for reuse by `haskell-doc-mode'."
                    (marker-position (process-mark proc))))
               (inferior-haskell-send-command proc (concat ":type " expr))
               ;; Find new point.
-              (goto-char (point-max))
               (inferior-haskell-wait-for-prompt proc)
+              (goto-char (point-max))
               ;; Back up to the previous end-of-line.
               (end-of-line 0)
               ;; Extract the type output
@@ -367,10 +424,11 @@ The returned info is cached for reuse by `haskell-doc-mode'."
                (save-excursion (goto-char parsing-end)
                                (line-beginning-position 2))
                (point))))))
-    (if (not (string-match (concat "\\`" (regexp-quote expr) "[ \t]+::[ \t]*")
+    (if (not (string-match (concat "^\\(" (regexp-quote expr) "[ \t\n]+::[ \t\n]*\\(.\\|\n\\)*\\)")
                            type))
         (error "No type info: %s" type)
-
+      (progn
+        (setf type (match-string 1 type))
       ;; Cache for reuse by haskell-doc.
       (when (and (boundp 'haskell-doc-mode) haskell-doc-mode
                  (boundp 'haskell-doc-user-defined-ids)
@@ -380,14 +438,14 @@ The returned info is cached for reuse by `haskell-doc-mode'."
         (let ((sym (match-string 1 type)))
           (setq haskell-doc-user-defined-ids
                 (cons (cons sym (substring type (match-end 0)))
-                      (remove-if (lambda (item) (equal (car item) sym))
-                                 haskell-doc-user-defined-ids)))))
+                      (delq (assoc sym haskell-doc-user-defined-ids)
+                            haskell-doc-user-defined-ids)))))
 
       (if (interactive-p) (message "%s" type))
       (when insert-value
         (beginning-of-line)
         (insert type "\n"))
-      type)))
+        type))))
 
 ;;;###autoload
 (defun inferior-haskell-info (sym)
@@ -404,8 +462,8 @@ The returned info is cached for reuse by `haskell-doc-mode'."
              (marker-position (process-mark proc))))
         (inferior-haskell-send-command proc (concat ":info " sym))
         ;; Find new point.
-        (goto-char (point-max))
         (inferior-haskell-wait-for-prompt proc)
+        (goto-char (point-max))
         ;; Move to previous end-of-line
         (end-of-line 0)
         (let ((result
@@ -436,6 +494,9 @@ The returned info is cached for reuse by `haskell-doc-mode'."
             (col (string-to-number
                   (match-string-no-properties 3 info))))
         (when file
+          (with-current-buffer (process-buffer (inferior-haskell-process))
+            ;; The file name is relative to the process's cwd.
+            (setq file (expand-file-name file)))
           ;; Push current location marker on the ring used by `find-tag'
           (require 'etags)
           (ring-insert find-tag-marker-ring (point-marker))
@@ -446,19 +507,19 @@ The returned info is cached for reuse by `haskell-doc-mode'."
 
 ;;; Functions to find the documentation of a given function.
 ;;
-;; TODO for this section: 
-;; 
+;; TODO for this section:
+;;
 ;; * Support fetching of local Haddock docs pulled directly from source files.
 ;; * Display docs locally? w3m?
 
 (defcustom inferior-haskell-use-web-docs
   'fallback
-  "Whether to use the online documentation. Possible values:
+  "Whether to use the online documentation.  Possible values:
 `never', meaning always use local documentation, unless the local
 file doesn't exist, when do nothing, `fallback', which means only
 use the online documentation when the local file doesn't exist,
 or `always', meaning always use the online documentation,
-regardless of existance of local files. Default is `fallback'."
+regardless of existance of local files.  Default is `fallback'."
   :group 'haskell
   :type '(choice (const :tag "Never" never)
                  (const :tag "As fallback" fallback)
@@ -466,9 +527,9 @@ regardless of existance of local files. Default is `fallback'."
 
 (defcustom inferior-haskell-web-docs-base
   "http://haskell.org/ghc/docs/latest/html/libraries/"
-  "The base URL of the online libraries documentation. This will
-only be used if the value of `inferior-haskell-use-web-docs' is
-`always' or `fallback'."
+  "The base URL of the online libraries documentation.
+This will only be used if the value of `inferior-haskell-use-web-docs'
+is `always' or `fallback'."
   :group 'haskell
   :type 'string)
 
@@ -478,11 +539,13 @@ only be used if the value of `inferior-haskell-use-web-docs' is
   :type 'string)
 
 (defcustom haskell-package-conf-file
-  (ignore-errors
-    (with-temp-buffer
-      (call-process "ghc" nil t nil "--print-libdir")
-      (expand-file-name "package.conf"
-                        (buffer-substring (point-min) (1- (point-max))))))
+  (condition-case nil
+      (with-temp-buffer
+        (call-process "ghc" nil t nil "--print-libdir")
+        (expand-file-name "package.conf"
+                          (buffer-substring (point-min) (1- (point-max)))))
+    ;; Don't use `ignore-errors' because this form is not byte-compiled :-(
+    (error nil))
   "Where the package configuration file for the package manager resides.
 By default this is set to `ghc --print-libdir`/package.conf."
   :group 'haskell
@@ -492,19 +555,17 @@ By default this is set to `ghc --print-libdir`/package.conf."
   "Fetch the module in which SYM is defined."
   (let ((info (inferior-haskell-info sym)))
     (unless (string-match inferior-haskell-module-re info)
-      (error 
-       "No documentation information available. Did you forget to C-c C-l?"))
+      (error
+       "No documentation information available.  Did you forget to C-c C-l?"))
     (match-string-no-properties 1 info)))
 
 (defun inferior-haskell-query-ghc-pkg (&rest args)
-  "Send ARGS to ghc-pkg, or whatever the value of
-`haskell-package-manager' is.  Insert the output into the current
-buffer."
+  "Send ARGS to `haskell-package-manager-name'.
+Insert the output into the current buffer."
   (apply 'call-process haskell-package-manager-name nil t nil args))
 
 (defun inferior-haskell-get-package-list ()
-  "Get the list of packages from ghc-pkg, or whatever
-`haskell-package-manager-name' is."
+  "Get the list of packages from `haskell-package-manager-name'."
   (with-temp-buffer
     (inferior-haskell-query-ghc-pkg "--simple-output" "list")
     (split-string (buffer-substring (point-min) (point-max)))))
@@ -544,9 +605,11 @@ buffer."
   ;; (expand-file-name "~/.inf-haskell-module-alist")
   (expand-file-name (concat "inf-haskell-module-alist-"
                             (number-to-string (user-uid)))
-                    temporary-file-directory)
+                    (if (fboundp 'temp-directory)
+                        (temp-directory)
+                      temporary-file-directory))
   "Where to save the module -> package lookup table.
-Set this to `nil' to never cache to a file."
+Set this to nil to never cache to a file."
   :group 'haskell
   :type '(choice (const :tag "Don't cache to file" nil) string))
 
@@ -644,10 +707,12 @@ we load it."
          (url (if (or (eq inferior-haskell-use-web-docs 'always)
                       (and (not (file-exists-p local-path))
                            (eq inferior-haskell-use-web-docs 'fallback)))
-                  (concat inferior-haskell-web-docs-base package "/" file-name)
+                  (concat inferior-haskell-web-docs-base package "/" file-name
+                          ;; Jump to the symbol anchor within Haddock.
+                          "#v:" sym)
                 (and (file-exists-p local-path)
                      (concat "file://" local-path)))))
-    (if url (browse-url url) (error "Local file doesn't exist."))))
+    (if url (browse-url url) (error "Local file doesn't exist"))))
 
 (provide 'inf-haskell)
 
